@@ -218,3 +218,46 @@ h2o.saveModelDetails <- function(object, path="", force=FALSE) {
   res$dir
 }
 
+#' Saves an existing Grid of models into a given folder.
+#'
+#' Returns a reference to the saved Grid.
+#'
+#' @param grid_directory A character string containing the path to the folder for the grid to be saved to.
+#' @param grid_id A chracter string with identification of the grid to be saved.
+#' @return Returns an object that is a subclass of \linkS4class{H2OGrid}.
+#' @examples
+#' \dontrun{
+#' library(h2o)
+#' h2o.init()
+#'
+#'iris.hex <- as.h2o(iris)
+#'
+#'ntrees_opts = c(1, 5)
+#'learn_rate_opts = c(0.1, 0.01)
+#'size_of_hyper_space = length(ntrees_opts) * length(learn_rate_opts)
+#'
+#'hyper_parameters = list(ntrees = ntrees_opts, learn_rate = learn_rate_opts)
+#'# Tempdir is chosen arbitrarily. May be any valid folder on an H2O-supported filesystem.
+#'baseline_grid <- h2o.grid("gbm", grid_id="gbm_grid_test", x=1:4, y=5, training_frame=iris.hex,
+#' hyper_params = hyper_parameters)
+#'
+#'grid_path <- h2o.saveGrid(grid_directory = tempdir(), grid_id = baseline_grid@grid_id)
+#'# Remove everything from the cluster or restart it
+#'h2o.removeAll()
+#'grid <- h2o.loadGrid(grid_path)
+#' }
+#' @export
+h2o.saveGrid <- function(grid_directory, grid_id){
+  params <- list()
+  params[["grid_directory"]] <- grid_directory
+  
+  url <- paste0("Grid.bin/", grid_id,"/export")
+  
+  res <- .h2o.__remoteSend(
+    url,
+    method = "POST",
+    h2oRestApiVersion = 3,.params = params
+  )
+  
+  paste0(grid_directory,"/",grid_id)
+}
